/*
   MindtelSocket.cpp

  Minimal socket interface for secure socket data transfer.

  */

#include <stdio.h>
#include <io.h>
#include <winsock2.h>
#include "MindtelSocket.h"
#include "log.h"

// windows specific garbage
static bool initialized = false;

static int init() {
	if ( !initialized ) {
		WORD wVersionRequested;
		WSADATA wsaData;
		int err;
 
		wVersionRequested = MAKEWORD( 2, 2 );
 
		err = WSAStartup( wVersionRequested, &wsaData );
		if ( err != 0 ) {
			/* Tell the user that we could not find a usable */
			/* WinSock DLL.                                  */
//			printf("WSAStartup didn't fucking work!\n");
			return(-1);
		}
 
		/* Confirm that the WinSock DLL supports 2.2.*/
		/* Note that if the DLL supports versions greater    */
		/* than 2.2 in addition to 2.2, it will still return */
		/* 2.2 in wVersion since that is the version we      */
		/* requested.                                        */
		if ( LOBYTE( wsaData.wVersion ) != 2 ||
			HIBYTE( wsaData.wVersion ) != 2 ) {
			/* Tell the user that we could not find a usable */
			/* WinSock DLL.                                  */
			WSACleanup( );
			return(-1); 
		}
 
	}
	initialized = true;
	return( 0 );
}

static void bzero( char* ptr, int size ) {
	for ( int i = 0; i < size; i++ ) {
		*ptr++ = 0;
	}
}


/*
 *  MindtelSocket::MindtelSocket
 *
 *  Purpose:
 *    Initialize the networking (windows only), then set up the socket.
 */

MindtelSocket::MindtelSocket() {
	if (init() != 0) {
		printf( "Sockets are fucked up on this machine.  Thanks, Microsoft!\n" );
		exit( 0 );
	}

	/* The WinSock DLL is acceptable. Proceed. */
}


/*
 *  MindtelSocket::Send
 * 
 *  Purpose:
 *    Send data out connected socket
 *
 *  Input parameters:
 *    buf -- pointer to memory to send out socket
 *    buflen -- how many bytes to send
 */

void MindtelSocket::Send( const char* buf, int buflen ) {
	send( sockfd, buf, buflen, 0 );
}


/*
 *  MindtelSocket::Close
 *
 *  Purpose:
 *    Close the socket connection
 */

void MindtelSocket::Close() {
	close( sockfd );
}


/*
 *  MindtelSocket::Recv
 * 
 *  Purpose:
 *    Recv data from connected socket
 *
 *  Input parameters:
 *    buf -- pointer to receive destination memory
 *    buflen -- how many bytes to receive
 */

int MindtelSocket::Recv( char* buf, int buflen ) {
	int result = recv( sockfd, buf, buflen, 0 );
	if ( result >= 0 ) {
		*(buf + result) = 0;  // for those callers that are string based
	}
	return( result );
}


/*
 *  MindtelSocketClient::MindtelSocketClient
 *
 *  Purpose:
 *    Constructor for side initiating a socket connection.
 *
 *  Input Parameters:
 *    ip -- ip address of the remote machine
 *    portNumber -- port to connect to
 */

MindtelSocketClient::MindtelSocketClient( char* ipin, int portNo)
: MindtelSocket() {
	memset( ip, 0, 30 );
	strncpy( ip, ipin, 29 );
	portNumber = portNo;
}


/*
 *  MindtelSocketClient::Connect( ip, portNumber ) 
 *
 *  Purpose:
 *    Set up TCP socket connection to a remote machine.
 *
 *  Return Value:
 *    true if the connection succeeds, otherwise false
 */

bool MindtelSocketClient::Connect() {
	sockfd = socket(AF_INET, SOCK_STREAM, 0);

	struct sockaddr_in servaddr;
	bzero((char*)&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(portNumber);
	servaddr.sin_addr.S_un.S_addr = inet_addr(ip);

    if (connect(sockfd, (const struct sockaddr*) &servaddr, sizeof(servaddr)) == -1) {
		return( false );
	} else {
		return( true );
	}
}


void MindtelSocketClient::GET( char* page ) {
	char buf[500];
	sprintf( buf, "GET %s HTTP/1.0\r\n\r\n", page );
	Send( buf, strlen( buf ));
}

int grabresult( char* s ) {
	while ( *s ) {
		if ( strncmp( s, "<int>", 5 ) == 0 ) {
			return( atoi( s + 5 ));
		}
		s++;
	}
	return( -1 );
}

int MindtelSocketClient::simpleXmlRpc( char* page, char* method, char* params ) {
	static char* bodyFormat = "<?xml version='1.0'?>\n<methodCall>\n<methodName>%s</methodName>\n<params>\n<param>\n<value><string>%s</string></value>\n</param>\n</params>\n</methodCall>\n";
	static char* headerFormat = "POST %s HTTP/1.0\r\nHost: nomatter\r\nUser-Agent: xmlrpclib.py/0.9.8 (by www.pythonware.com)\r\nContent-Type: text/xml\r\nContent-Length: %d\r\n\r\n";
	char sendStuff[1000];
	sprintf( sendStuff, bodyFormat, method, params );
	int len = strlen( sendStuff );
	char headerStuff[300];
	sprintf( headerStuff, headerFormat, page, len );
	Send( headerStuff, strlen( headerStuff ));
	Send( sendStuff, strlen( sendStuff ));
				char result[1000];
				while ( Recv( &result[0], 999 ) > 0 ) {
					int x = grabresult( &result[0] );
					if ( x != -1 ) {
						return( x );
					}
					//printf( result );
				}
				//printf( "\nThat was it!\n" );
	return( 0 );
}


/**
 *  MindtelSocketServer::MindtelSocketServer
 *
 *  Constructor for a simple server class.
 */
MindtelSocketServer::MindtelSocketServer( int port ) : MindtelSocket() {
	/* The WinSock DLL is acceptable. Proceed. */

    listenfd = socket(AF_INET, SOCK_STREAM, 0);

    bzero((char*)&servaddr, sizeof(servaddr));
    servaddr.sin_family      = AF_INET;
    servaddr.sin_addr.s_addr = htonl( INADDR_ANY );
    servaddr.sin_port        = htons( port );

	//log_message( "calling bind");

    bind(listenfd, (const struct sockaddr*)&servaddr, sizeof(servaddr));

	//log_message( "calling listen" );
	listen( listenfd, 5 );
}

bool MindtelSocketServer::AcceptConnection() {
	int clilen = 0;
	bzero( (char*)&cliaddr, sizeof( cliaddr ));
    clilen = sizeof(cliaddr);
	printf( "accept connection, listenfd is %d\n", listenfd );
	if ( (sockfd = accept(listenfd, (struct sockaddr*)&cliaddr, &clilen)) < 0) {
		log_error( "accept error" );
		return( false );
	}
	return( true );
}

#include <process.h>
extern bool doPost( char* pageToGet, char* buf, MindtelSocketServer* theServer );

void MyThreadProc( void* x ) {
	printf( "Thread beginning\n");
	// make internal to thread copy
	MindtelSocketServer threadInternal = *(MindtelSocketServer*)x;

	while ( 1 ) {
			char buf[5000];
			buf[0] = 0;
			int recvResult = threadInternal.Recv( buf, 5000 );
			if ( recvResult <= 0 ) {
				break;
			}
			printf( "\n\nReceived (%d): '''%s'''\n\n", recvResult, buf );
			char* pageToGet = NULL;
			char* command = &buf[0];
			for ( int i = 0; i < 100; i++ ) {
				if ( buf[i] == ' ' ) {
					buf[i] = 0;
					if ( pageToGet == NULL ) {
						pageToGet = &buf[i+1];
					}
				}
			}
			if (( strcmp( command, "GET" ) == 0 ) && ( pageToGet != NULL )) {
				threadInternal.GetPage( pageToGet );
			} else if ( strcmp( command, "POST" ) == 0 ) {
				if ( !doPost( pageToGet, buf, &threadInternal ) ) {
					char pageBuf[100];
					strcpy( pageBuf, pageToGet );
					recvResult = threadInternal.Recv( buf, 5000 );
					if ( recvResult <= 0 ) {
						break;
					} else {
						doPost( pageBuf, buf, &threadInternal );
					}
				}
			}
	}
	printf( "Thread ending\n" );
}

void MindtelSocketServer::Run() {
	while ( 1 ) {
		printf( "Waiting for a connection\n" );
		if ( AcceptConnection() ) {
			_beginthread(MyThreadProc, 10000, this);

			//Close();
		}
	}
}
        
void MindtelSocketServer::SendPage( char* text ) {
	char header[500];
	char body[500];
	static char* headerFormat = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n";
	static char* bodyFormat = "<HTML><HEAD><TITLE>TEST</TITLE></HEAD><BODY><H3>%s</H3></BODY></HTML>";
	sprintf( body, bodyFormat, text );
	sprintf( header, headerFormat, strlen( body ));
	Send( header, strlen( header ) );
	Send( body, strlen( body ) );
}

static char* headerFormat = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\nContent-Length: %d\r\n\r\n";

void MindtelSocketServer::Redirect( char* location ) {
	char header[500];
	char body[500];
	static char* bodyFormat = "<html><head><meta HTTP-EQUIV=\"REFRESH\" CONTENT=\"0; URL=%s\"></head></html>";
	sprintf( body, bodyFormat, location );
	sprintf( header, headerFormat, strlen( body ));
	Send( header, strlen( header ) );
	Send( body, strlen( body ) );
}

#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

void MindtelSocketServer::GetPage( char* page ) {
	printf( "\n\nGetting page %s\n", page );
	char filePath[500];
	sprintf( filePath, ".%s", page );
	int fh = -1;
	int fileLength = 0;
	fh = _open( filePath, _O_RDWR, _S_IREAD );
	if ( fh != -1 ) {
		fileLength = _filelength( fh );
		_close( fh );
	}
	FILE* fp = fopen( filePath, "rb" );
	if (( fileLength > 0 ) && ( fp != NULL )) {
		char header[500];
		sprintf( header, headerFormat, fileLength);
		Send( header, strlen( header ));
		while ( fileLength > 0 ) {
			char data[10000];
			int count = fread( &data[0], 1, 10000, fp );
			if ( count > 0 ) {
				Send( &data[0], count );
				fileLength -= count;
			} else {
				break;
			}
		}
		fclose( fp );
		//Close();
	} else {
		printf( "Hey, what is this!\n" );
	}
	printf( "I got it\n" );
}